import sys
sys.path.append('../src')
import pygame
import unittest
import sign

# Dummy Vehicle Class
class Vehicle():
    def __init__(self):
        self.isWaiting = False

    # Set the vehicle's waiting status
    def setWaiting( self, waiting ):
        self.isWaiting = waiting


class TestSignFunctions(unittest.TestCase):
    # Set up elements needed for testing sign
    def setUp(self):
        # Create a new traffic light object
        self.mySign = sign.TrafficLight(2, 4)
        self.mySign.redLight = True
        
        # Create a new vehicle to be tested
        self.myVeh = Vehicle()

    # Test handleVehicle
    def testhandleVehicle(self):
        # self.sign.redLight = True
        # Vehicle's isWaiting should be true
        self.mySign.handleVehicle(self.myVeh)
        self.assertTrue(self.myVeh.isWaiting, "Vechicle's isWaiting should be True")
        
        # Now self.sign.redLight = False
        # Vechile's isWaiting should be false
        self.mySign.redLight = False
        self.mySign.handleVehicle(self.myVeh)
        self.assertFalse(self.myVeh.isWaiting, "Vechicle's isWaiting should be False")

    # Test update
    def testupdate(self):
        # Set the traffic light to red
        self.mySign.redLight = True

        # Clock to count the looping time
        myClock = pygame.time.Clock()
        myClock.tick()
        # How long the loop has run
        currentTime = 0

        # Test checkpoints
        test1 = False
        test2 = False
        test3 = False

        # Game loop dummy
        while(True):
            # Update the traffic light
            self.mySign.update()

            # Tests
            if(currentTime > 0 and currentTime < 2000 and not test1):
                self.assertTrue(self.mySign.isRed(), "MySign's isRed should be True")
                test1 = True
            if(currentTime > 2000 and currentTime < 6000 and not test2):
                self.assertFalse(self.mySign.isRed(), "MySign's isRed should be False")
                test2 = True
            if(currentTime > 6000 and currentTime < 8000 and not test3):
                self.assertTrue(self.mySign.isRed(), "MySign's isRed should be True")
                test3 = True

            # Update loop time
            currentTime += myClock.tick()
            # Adjust frame rate so it mimics the real loop (not really needed here)
            pygame.time.wait(10)

            # Done with testing
            if( currentTime > 10000 ):
                break
                

if __name__ == '__main__':
    suite = unittest.TestLoader().loadTestsFromTestCase(TestSignFunctions)
    unittest.TextTestRunner(verbosity=2).run(suite)